#include "defs.h"

char txt1[] = "AD7328-Data";

uint16_t write2bytes(uint16_t command) {
uint8_t higherByte, lowerByte;
uint16_t result;

         higherByte = (uint8_t)((command & 0xFF00) >> 8);
         lowerByte = (uint8_t)(command & 0x00FF);

         Chip_Select = 0;

         SPDR = higherByte;  // sending high byte first
         while(!(SPSR & (1<<SPIF)));
         result = SPDR & 0xFF;
         result <<= 8;
         SPDR = lowerByte;  // sending low byte
         while(!(SPSR & (1<<SPIF)));
         result |= SPDR;

         Chip_Select = 1;

         return result;
}//write2bytes

void setRangeRegister(uint8_t ch, uint8_t range) {
static uint16_t range_ = 0xA000;

     if ((ch < 0) || (ch > 3)) return;

     switch (ch) {
       case 0:
         range_ &= CH0_MASK;
         range_ |= range << 11;
       break;
       case 1:
         range_ &= CH1_MASK;
         range_ |= range << 9;
       break;
       case 2:
         range_ &= CH2_MASK;
         range_ |= range << 7;
       break;
       case 3:
         range_ &= CH3_MASK;
         range_ |= range << 5;
       break;
       default: break;
     }//switch

     Chip_Select = 0;
     write2bytes(range_);
     Chip_Select = 1;

     return;
}//setRangeRegister

// SPI initialize @ clock rate of 250 kHz
void SPI_init(void) {

     DDB6_bit = 0;              // Set PB6 pin as input (MISO)
     DDB5_bit = 1;              // Set PB5 pin as output (MOSI)
     Chip_Select = 1;           // Deselect ADC
     Chip_Select_Direction = 1; // Set CS# pin as Output
     SPI1_Init();

     Lcd_Init();                // Initialize LCD
     Lcd_Cmd(_LCD_CLEAR);       // Clear display
     Lcd_Cmd(_LCD_CURSOR_OFF);  // Cursor off

     return;
}//SPI_init

void setControlRegister(uint8_t ch, uint16_t setup) {

     if ((ch < 0) || (ch > 7)) return;

     setup |= ((ch & 0x07) << 10);
     Chip_Select = 0;
     write2bytes(setup);
     Chip_Select = 1;

     return;
}//setControlRegister

void ADC7328_init(void) {
uint8_t i = 0;

     for (i = 0; i < 4; i++) setRangeRegister(i, BIP10); // Fill in both range registers
     setControlRegister(0, SETUP_1);                     // Control register
     write2bytes(0xFE00);                                // Sequence register (no kidding)

     return;
}//ADC7328_init

void main() {
uint16_t buffer[8], result = 0;
//float bufferF[8];

     SPI_init();
     ADC7328_init();
     Lcd_Out(2, 1, txt1);

     while (1) {
        result = write2bytes(0x0000) & 0x1FFF; // solely get least 13 bits
        if ((result & 0x1000) >> 12) {// bit12 = 1
           sprintf(buffer, "%4u", result & 0x0FFF);
           Lcd_Out(1, 1, "+");
           Lcd_Out(1, 2, buffer);
        }//if
        else {
           sprintf(buffer, "%4u", (~result) & 0x0FFF);
           Lcd_Out(1, 1, "-");
           Lcd_Out(1, 2, buffer);
        }//else
//        sprintf(bufferF, "%1.4f", 0.001222 * (float) result - 0.002587);
     }//while

     return;
}